;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PIC16F676. This file contains the basic code               *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    ZU676_61.asm                                           *
;    Date:           11-12.2015                                                 *
;    File Version:   v6.1                                                 *
;                                                                     *
;    Author:       NickF                                                   *
;    Company:      MyCompaine                                                   *
;                                                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P16F676.INC                                      *
;                                                                     *
;**********************************************************************
;                                                                        *
;    Notes:                                                             *
;                                                                     *
;**********************************************************************

	list      p=16f676            ; list directive to define processor
	#include <p16F676.inc>        ; processor specific variable definitions
	#include <macro61.inc>
	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.

;      
;==============================================
	CBLOCK 0x20			
D6,D5,D4,D3,D2,D1,D0,r2,r1,r0,Binc,Decn
TEMP2,TEMP1,TEMP0,NumH,NumL,t_NumH,t_NumL
znm_new,znm_old,t_znm,znak,zn_dac,n_chan
t_chan,t_nk,cyc0,cycle,cycle1,cycle2
rejim,fl_rej,count,TxByte,fla_TR
cyc_sim,cyc_but					
		ENDC
;==============================================
; "3" -   
; "2" -   
; "1" -     
;  "fl_rej"
; "3"-    "3", "0" - Me_hi, "1" - Li_ion
; "2"-    "2", "0" - , "1" - 
; "1"-    "1", "0" -  , "1" -   
; "0"-    "0", "0" -  , "1" -   
; "0-3"    "1"
; "4" -     "0"   "1"
; "5" -   - "1",  - "0"
; "6" - 
;---------------------------------------------------------
;rejim -     "0,1" 
;0 -   /  
;1 -   /  
;2 -  / 
;3 - Hi / Li
;znm_new -      
;znm_old -    
;n_chan -  ADCON0,   
;cycle -     16.384 * 3 = 49.044 
; "cycle" = 6    - 49.044 * 6 = 294.264 c
;cycle1 -     "cycle1" = 3 
; = 294,264 * 3 = 882.8 
;cycle2 -    RS232 - 882.8 * 68 = 60.03 
;znak - 7-  
;count,t_nk,t_znm -      /
;cyc0 -   TMR0,     - 16.384 vc
;cyc_sim -     "X--"
;cyc_but -     
;NumH     ADRESH, NumL    ADRESL
;r2,r1,r0 -    24-  / 
;Binc,Decn,TEMP2,TEMP1,TEMP0,t_NumH,t_NumL -   
;/    HEX->DEC

;**********************************************************************
		ORG     0x000             ; processor reset vector
		goto    main              ; go to beginning of program

		ORG     0x004             ; interrupt vector location
		goto	ObrInt
		org		0x007
; Segm7 :    ->  
;    ,   W=07h -  "7"
Segm7		
    CLRF   PCLATH
	ADDWF  PCL, F
;		 0	  1	   2	3	 4	  5	   6	7	 8	  9	   A	b	 C	  d	   E	F
	dt	07E, 00C, 0B6, 09E, 0CC, 0DA, 0FA, 00E, 0FE, 0DE, 0EE, 0F8, 072, 0BC, 0F2, 0E2 
;-    -  
;================================================================
; 1    W=0 -  
Tab1
	CLRF   PCLATH
	ADDWF  PCL, F	;sim	adr	Tab1
	dt	07C,81h		;"U", 	0		
	dt	08C,85h		;"-I"	2		 
	dt	00C,85h		;"I"	4		 
	dt	0F0,89h		;"t"	6		
	dt	09E,91h		;"3"	8		
	dt	0E6,91h		;"P"	0ah		
	dt	0EC,91h		;"H"	0ch		Hi
	dt	070,91h		;"L"	0eh		Li
	dt	080			;"-"	10h		

;===============================================
;    
;===============================================
ULz			equ		0x035b	;Uzap max = 4.20 -    Li-I
ULp			equ		0x0266	;Upaz min = 3.00 -    Li-I 
UHz			equ		0x0133	;Uzap max = 1.50 -    Me-Hi 
UHp			equ		0x00cd	;Upaz min = 1.00 -    Me-Hi 
abs_grad	equ		H'022f'		;  2730 
por_grad	equ		H'0052'		;  40
koefU		equ		H'1317'		;    1000 - 4887
;===============================================
; 

Brate		equ		.30		;   / "Transmit"
TDelay		equ		8ch
c_sim		equ		.16		;   
c_but		equ		.8		;    / 
cy_			equ		6		;   "cycle"
cy_1		equ		3		;   "cycle1"
cy_2		equ		.68		;   "cycle2"
;*******************************************************************

main
		; init PORTA & SFR

		bcf 	STATUS,RP0 			;Bank 0
		bsf		STATUS,NOT_TO		;reset on POR
		bsf		STATUS,NOT_PD
		clrf 	PORTA 				;Clear PORTA
		movlw 	 7					;Set RA<2:0> to 
		movwf 	CMCON 				;comp-off digital I/O
		bsf 	STATUS,RP0 			;Bank 1
		bsf		PCON,NOT_POR		;enable POR
		clrf 	ANSEL 				;digital I/O
		clrf 	WPUA 				;  
		movlw 	01fh
		movwf 	TRISA				;Set PORTA<3..0> as input
		movlw 	07h 				;
		movwf 	ANSEL 				;Set PORTA<3..0>  RA<3>  analog input
		clrf 	PIE1 				;   		

;init PORTC
		bcf 	STATUS,RP0 			;Bank 0
		clrf	PORTC
		bsf 	STATUS,RP0 			;Bank 1
		clrf 	TRISC 				;set PORTC as outputs
		bcf 	STATUS,RP0 			;Bank 0

; these first 4 instructions are not required if the internal oscillator is not use
		call    0x3FF             ; retrieve factory calibration value
		bsf     STATUS,RP0        ; set file register bank to 1 
		movwf   OSCCAL            ; update register with factory cal value 
		bcf     STATUS,RP0        ; set file register bank to 0

; ADC,TMR0
	banksel		ADCON1			
		movlw	030h			;    
		movwf 	ADCON1			;RC  ~ 8 us.
		clrf	TMR0			;  "0"
		movlw	85h				; - 
	banksel		OPTION_REG		;   1/8 
		movwf	OPTION_REG		; CLK  TMR0 - 8 ,   "T0CS"=0
;   256*64= 16.384 .

; remaining code goes here
start
;  020h-04fh
		movlw	020h			;   
		movwf	FSR
	
c_ind	clrf	INDF			;      FSR
		incf	FSR,f			; FSR
		movf	FSR,w
		sublw	60h				;FSR  60h? 
		skpz
		goto	c_ind		;,     
;====================================
;  
		movlw	81h
		movwf	n_chan
		movlw	1
		movwf	cyc0		;  TMR0
		movwf	znm_new		; 
		movlw	c_sim
		movwf	cyc_sim		;   
		movlw	c_but		;   
		movwf	cyc_but		; / "Button"
;==========================================================
; 
start_cycle2
		movlw	cy_2
		movwf	cycle2
;==========================================================
;		btfsc	fl_rej,4
;		goto	start_cycle1
;      
;      
;		bsf		fl_rej,4
		movlw	1
		xorwf	rejim,f		;  
	combit	0,rejim,volt	;  : 0 -"U", 1 -"t"
		bsf		fl_rej,1	;  
		movlw	89h
		movwf	n_chan
		goto	clr_sim
volt
		bcf		fl_rej,0	;  
		movlw	81h
		movwf	n_chan
clr_sim
;		bcf		fl_rej,4	;  
		call	AdcPusk
start_cycle1
		movlw	cy_1			
		movwf	cycle1	
start_cycle
		movlw	cy_
		movwf	cycle		;    "CBLOCK"
;=============================================================
set_int
;  020h-02fh
clear_mem
		movlw	020h			;   
		movwf	FSR
clear
		clrf	INDF			;      FSR
		incf	FSR,f			; FSR
		movlw	TEMP0+1
		subwf	FSR,w			;FSR  35h? 
		skpz
		goto	clear			;,     
; 
		bcf 	STATUS,RP0 		;Bank 0
		clrf	INTCON
		bsf		INTCON,PEIE		;   
		bsf		INTCON,T0IE		;    TMR0
		bsf		INTCON,GIE		; 

; 1- 
;=====================================================================
	combit	4,fl_rej,sdig	;  ?    
;,  
; 2  3 
	comreg	1,znm_new,_sim
		goto	Blok_sim
_sim	movlw	10h			;  "-"  2  3 
		call	Tab1
		goto	st_zn
;   
;========================================================================
Blok_sim
		clrf	t_znm
		rejfla				;     
		andwf	fl_rej,w	;    1
		skpz				;  - 0  1
		goto	bs1			;   
		goto	bs0			; "Tab1"
bs1		bsf		t_znm,0		; 1
bs0		clrc				; 0
		movfw	rejim		;  -> W
		movwf	count		;   
		rlf		t_znm,f		;t_znm = t_znm * 2
		rlf		count,f		
		rlf		count,w		;W = count * 4
		addwf	t_znm,w		;W = (count * 4) + (t_znm * 2) -   "Tab1" 
		movwf	count		;   . "count"
		addlw	1
		call	Tab1		;     "ADCON0"
		movwf	n_chan		;write number channel to "n_chan"
		movfw	count
		call	Tab1		;   "Tab1"
		goto	st_zn		;     "znak"
;==============================================================================
sdig
		call	Comp_Por
;==============================================================================
;      4-   
		call	Mult14_10		;   . 4887
		call	BIN_DEC			; /  HEX -> DEC
; 2  /      
;==============================================================================
	rej_bit	1,spU,sme		; ?
		goto	sme
spU
	rej_bit	0,st0,st0		;     
sme		movf	D5,w		; ,  "0123"  "123"		
		movwf	D6
		movf	D4,w	
		movwf	D5
		movf	D3,w	
		movwf	D4
st0		clrc				; "D3"
		movlw	5
		subwf	D3,w		; "D3" => 5,   1   
		skpc
		goto	st1
		movlw	9
		subwf	D4,w
		skpz
		incf	D4,f			
;    			
;============================================================
st1		movfw	znm_new
		addlw	1fh		;   
		movwf	FSR
		movf	INDF,w		;      
;      1  2
;=============================================================
st_seg
		call	Segm7		;/    7  
;      
st_zn
		movwf	znak		;akk->znak
;================================
;  
		movlw	0FFh
		movwf	TEMP0
		movlw	0E0h
		movwf	TEMP1
		call	Shift_16bit
		movlw	.10
;================================
		call	Pause
		call	WriteZnak	;/    
;====================================================	
loop	;  16 	
		movf	cyc0,f
		skpz
		goto	loop
;============================================================
		decfsz	cyc_but,f	;  16   "Button"
		goto	loo
		call	Button		; /  
		movlw	c_but
		movwf	cyc_but
;***********************************
;   
		btfss	fl_rej,4	; ?	
		goto	loo			;, 	
		decfsz	cyc_sim,f	;  
		goto	loo			;
		bcf		fl_rej,4	;  
;***********************************
loo		movlw	1	
		movwf	cyc0
		incf	znm_new,f
	combit	2,znm_new,ldec
		movwf	znm_new			; 1  a
		call	AdcPusk
		movlw	.30
		call	Pause
		decfsz	cycle,f		
ldec	goto 	set_int		;cycle = 6, time = 49 ms
		decfsz	cycle1,f	;      
	goto	start_cycle		;cycle1 = 3, time = 6 * 49 = 294 ms
;=================================================================================
		decfsz	cycle2,f	;     
							;  
		goto	start_cycle1
	
;  "U","I","t"
;===============================================================
		movfw	n_chan	;    "t_chan"
		movwf	t_chan
		movlw	81h
		movwf	n_chan
		call	Adc_Trans	; "U"   
		movlw	89h
		movwf	n_chan
		call	Adc_Trans	; "t"   
		movlw	85h
		movwf	n_chan
		call	Adc_Trans	; "I"   
		movlw	21h
		movwf	TxByte
		call	Transmit
		movfw	t_chan
		movwf	n_chan	;    "n_chan"
		goto	start_cycle2
;===============================================================
;****************************
;
;****************************
;/     LED 
;  /   znak 
;7-      
WriteZnak
		clrf	t_nk
		movfw	znm_new
		movwf	t_znm		; "znm_new"  "t_znm"
	rej_bit	0,w2,w1			;     
		goto	ww1			; ,   
w1	comreg	1,znm_new,ww
		bsf		znak,0		;1- ,  
		goto	ww
w2	rej_bit	1,ww,ww1		; 
	comreg	2,znm_new,ww
		bsf		znak,0		;2- ,  
ww		btfsc	fl_rej,4
ww1		bcf		znak,0		;   ,  
;==========================================================			
;   () 
;==============================================	
		bsf		STATUS,C		
shi		rrf		t_nk,f		;     
		decfsz	t_znm,f		;     
		goto	shi
		movlw	0e0h
		xorwf	t_nk,f		; Q5..Q7
;  "znak"   "zn_dac"     
;==================================
; 7-   ()  TEMP0
		movf	znak,w
		movwf	TEMP0
		comf	TEMP0,f
;==================================
;  "zn_dac"   
;=============================================
		movlw	1fh
		andwf	zn_dac,w
		addwf	t_nk,w
		movwf	zn_dac
		movwf	TEMP1
	combit	2,fl_rej,s0_r4		; "2"  1?
		bsf		PORTC,4			;,  "".   
		goto	sdv
s0_r4
		bcf		PORTC,4			;,  "".   
sdv		call	Shift_16bit		
		return

;===============================================		
;/       
;===============================================
Sound
		bsf		PORTC,5
	Delay	.100
		bcf		PORTC,5		
		return
;=============================
Pause
		movwf	TEMP0
pa		decfsz	TEMP0,f		
		goto	pa 
		return
;=============================
;/  
Button
		btfsc	PORTA,3		; ? 
		return				;,  
		btfsc	fl_rej,5	;  ?
		goto	kni			;,   
		bsf		fl_rej,5	;,    
		movfw	znm_new		;   
		movwf	znm_old		;
		return
kni		;  
		movfw	znm_new
		subwf	znm_old,w	;   ?
		skpnz				;, 			
		goto	knm
		goto	knp				
knm		bsf		fl_rej,4	;  
		call	Sound	
kn1	comreg	1,znm_new,kn2
		inv_bit_fl			;     
		goto	knp
;=================================================================
kn2
	comreg	2,znm_new,kn3	
		bcf		fl_rej,4	;  
	combit	0,fl_rej,kn_ret		;,   ? , 
;  
;=================================================================

		movf	zn_dac,w
		movwf	t_nk
		movlw	1fh			;  
		andwf	t_nk,f
		incf	t_nk,f		; 
	combit	5,t_nk,knz		;,   ?
		bcf		t_nk,5		;,  
knz
		movlw	0e0h		;  7..5 
		andwf	zn_dac,f
		movf	t_nk,w
		addwf	zn_dac,f
		goto	knp
kn3
		incf	rejim,f		;  
		btfss	rejim,2		;   4?
		goto	knp			;, 
		bcf		rejim,2		;,  
knp
		bcf		fl_rej,5	;  
		call	Sound
kn_ret	
		movlw	c_sim
		movwf	cyc_sim	
		return
;================================================
;/  
Zero_Cur
		movlw	0e0h
		andwf	zn_dac,f
		return		
;=============================================
;/  24  HEX   
; 7  DEC 
;=============================================
BIN_DEC
		MOVLW	.24       ; 
		MOVWF	Binc
BITLP
		RLF		r0,F     ; hex-   r0 
		RLF		r1,F     
		RLF		r2,F
		MOVLW	D0
		MOVWF	FSR
		MOVLW	7        ;   
		MOVWF	Decn
		MOVLW	6        ;   6   
;For each bit the inner loop is repeated 8 times, 
;with shift in of the next bit, "times 2" and DecAdj of each digit
ADJLP
		RLF		INDF,F     ; 2*digit, then shift in "next bit for "nd0" 
;or else the carry from the previous digit
		ADDWF	INDF,F   ; Add 6, clears Cf and gives 1 in bit 4 if the
		BTFSS	INDF,4   ; addition is needed; zero if not, when
		SUBWF	INDF,F   ; we subtract it again. Sets Cf
		BSF		STATUS,C   ; Cf could be 0 or 1, so make it 1 as default
		BTFSS	INDF,4		;Bit 4 is the carry to the next digit
		BCF		STATUS,C   ; Reset Cf to zero if bit 4 is clear
		BCF		INDF,4     ; For BCD clear bit 4 in case its one
		decf	FSR,F     ; Go to next digit, (Cf not affected)
		DECFSZ	Decn,F 		;End of inner loop. check digit count and
		GOTO	ADJLP     ; round again if its not zero
		DECFSZ	Binc,F		;End of outer loop, one pass through digits,
		GOTO	BITLP     ; check bit count and repeat if necessary.
		RETURN
;=================================================
; 14bit by 10bit unsigned multiply
;  by Martin Sturm 2010
; tested over all 14, 10bit input combinations
;
; Useful for 10bit ADC and result that fit in 24bits
;
;  a (14bit) = aH:aL  (not modified)
;  b (10bit) = bH:bL  (not modified)
;  r3:r2:r1 is 24-bit result
;
; ignores non-zero bits above 10th for b,
; incorrect result if a has non-zero bits above the 14th
;  unless optional ANDLW are used
;
;  r = a*b = r3:r2:r1
;  r = 2^8*(aH:aL * bH) + (aH:aL * bL)
;             14x2            14x8
;
; 83 instructions, 71-83 cycles, 77 avg
;/  NumH,NumL  . 4887
;=================================================
Mult14_10
	; r3:r2:r1 = TEMP1:TEMP0 * NumL
	; 65 cycles
		movlw	high koefU
		movwf	TEMP1
		movlw	low koefU
		movwf	TEMP0

		CLRF	r2
		CLRF	r1
		CLRF	r0
		CLRC
	mul		TEMP0,8,r0

	mul		TEMP1,6,r1

		RRF	r2,F
		RRF	r1,F
		RRF	r2,F
		RRF	r1,F
; carry always clear by here
; r3:r2 += TEMP1:TEMP0 * NumH
; 6-18 cycles, 12 avg
	combit	0,NumH,g1
		MOVFW	TEMP0
		ADDWF	r1,F
		SKPNC
		INCF	r2,F
		MOVFW	TEMP1
;	ANDLW	0x3F		;(optional) if TEMP1 has nonzero bits above 14th
		ADDWF	r2,F		;never sets carry
g1
	combit	1,NumH,g2
		RLF		TEMP1,W		; carry always clear before here
;	ANDLW	0x7E		;(optional) if TEMP1 has nonzero bits above 14th,
;	CLRC				;and, clear any carry
		ADDWF	r2,F		;never sets carry
		RLF		TEMP0,W		;W = (2*TEMP0 & 0xFF)
		ADDWF	r1,F
		SKPNC
		INCF	r2,F
		BTFSC	TEMP0,7		;9th bit of 2*TEMP0
		INCF	r2,F
g2
		return

;==============================================
;/  16-   16 
;  "TEMP0",     "TEMP1"
Shift_16bit
		call	Shift_8bit		;   TEMP0   U2
		movf	TEMP1,w		;   TEMP1  TEMP0
		movwf	TEMP0
		call	Shift_8bit		;    U2   U5
;     TEMP0   U2
;     U2  U5
;ST_SP        16-  
		bsf		PORTC,2
		nop
		nop		
		bcf		PORTC,2
		return

Shift_8bit
		movlw	.8
		movwf	count		;   
per
		clrc	
		rlf		TEMP0,f
		skpnc
		goto	seg1		
		bcf		PORTC,0		; =0
		goto	seg0
seg1	bsf		PORTC,0		; =1
seg0	bsf		PORTC,1		;  
		nop
		nop
		bcf		PORTC,1		;  
		DECFSZ	count,f		;  
		goto	per			;    
		bcf		PORTC,0		;  0  
		return
;=========================================
;/      
;=========================================
AdcPusk
		banksel	PIE1			;     
		bsf		PIE1, ADIE 		;   
		banksel	ADCON0			;   ADCON0
		movfw	n_chan		;  : n_chan=81h - AN0,
;n_chan=85h - AN1, n_chan=89h - AN2
		movwf 	ADCON0		
		bsf		ADCON0,GO		; ADC	
		return
;==========================================
;/  
;==========================================
Transmit
		banksel	INTCON	
		bcf		INTCON,GIE
		bsf		fla_TR,0		;   
; "0" = 1 -  ,  "0" = 0 -  
		movlw 	8       		; 8 
		movwf 	count
		bcf 	PORTA,5			;  = 0
next_Tx
		movlw	Brate			; 24 
		call	Pause			;  1/4  			
 		btfss	 TxByte,0       ; "0" = 1
		goto	t0				;,    "0" 
		bsf		PORTA,5			;  "1"	
		goto 	t1   
t0		bcf		PORTA,5			;set TxD -> "0"	
t1		rrf 	TxByte,f		;    
		decfsz 	count,f			; "count"  "0",  
		goto 	next_Tx			;
		movlw	Brate			; 24 
		call	Pause			;  1/4  
		bsf PORTA,5				;  
		movlw	Brate			; 24 
		call	Pause			;  1/4  	     
		clrf	TxByte			;  
		bcf		fla_TR,0		; 
		banksel	INTCON	
		bsf		INTCON,GIE		; 
		return
;=======================================
;/  "U","I","t"     -
;=======================================
Adc_Trans
		movfw	NumH		;  
		movwf	t_NumH
		movfw	NumL
		movwf	t_NumL
		call	AdcPusk		; 
		movlw	20h
		call	Pause
		movfw	NumH
		movwf	TxByte
		call	Transmit	; .  	
		movlw	20h
		call	Pause
		movfw	NumL
		movwf	TxByte
		call	Transmit	; .  
		movlw	20h
		call	Pause
		movfw	t_NumH		;  
		movwf	NumH
		movfw	t_NumL
		movwf	NumL
		return

;=============================================================
;/     /
;=====================================================================
Comp_Por
	comreg	0,rejim,stemp	;  "stemp"   		
;    "0"
	combit	0,fl_rej,stU	; ?
		goto	cp_ret		; ,   
;  ,   
stU
	combit	2,fl_rej,sZ		; ""?
	combit	3,fl_rej,sHrh	;,  "Li" ?
;     
;============================================
;,  .    Li
	Umin	high ULp,NumH,sLrl,cp_ret
sLrl
; .    Li
;============================================
	Umin	low ULp,NumL,cp_ret,cp_ret
sHrh
;============================================
; .    Hi
	Umin	high UHp,NumH,sHrl,cp_ret
sHrl
; .    Hi
;============================================
	Umin	low UHp,NumL,cp_ret,cp_ret
;     
;============================================
sZ							; 
	combit	7,rejim,sHzh	; "Li" ?
;============================================
;,  .    Li
	Umax	high ULz,NumH,cp_ret,sLzl
sLzl
; .    Li
;============================================
	Umax	low ULz,NumL,cp_ret,cp_ret
sHzh
; .    Hi
;============================================
	Umax	high UHz,NumH,cp_ret,sHzl
sHzl
; .    Hi
;============================================		
	Umax	low UHz,NumL,cp_ret,cp_ret
;============================================

;     
;====================================================
stemp
	comreg	1,rejim,cp_ret		;,  "rejim"   1		
	combit	1,fl_rej,cp_ret		;,    
; 
; 
		clrc
		movlw	high abs_grad	;  2.73  
		subwf	NumH,w			; 
		skpc					; NumH => 2, 
		goto	cp_ret			;   
		movwf	NumH			;    NumH
		movlw	low abs_grad
		subwf	NumL,f			; NumL => 30h
		skpc
		goto	zaem
		goto	por				;    
zaem
		movf	NumH,f
		skpnz
		goto	cp_ret
		decf	NumH,f			;  1  NumH
;     
;=============================================
por		clrc
		movlw	por_grad	; 40 
		subwf	NumL,w		;    
		skpc				;NumL => 40,     
		goto	cp_ret		;NumL < 40 
		call	Zero_Cur	; 
cp_ret
		return
;==========================================
;/  
;==========================================
ObrInt
		movwf	05eh
		movf	STATUS,w
		movwf	05fh	
	banksel		INTCON			;
		bcf		INTCON,GIE		;  
	banksel		PIE1
	combit	ADIE,PIE1,it		;   ? ,   TMR0
	banksel		PIR1			;,      
	combit	ADIF,PIR1,it		;  ? ,   TMR0
	banksel		PIE1			
		bcf		PIE1, ADIE 		;   
	banksel		PIR1
		bcf		PIR1,ADIF		;  ADIF
	banksel		ADRESL
		movf	ADRESL,w
		movwf	NumL			; . 
	banksel		ADRESH
		movf	ADRESH,w
		movwf	NumH			; . 
		goto	isr
it
	banksel		INTCON
	combit	T0IE,INTCON,isr		;  TMR0 ? ,   "isr"
								
	combit	T0IF,INTCON,isr		;  TMR0? ,   "isr"
								
		bcf		INTCON,T0IF		;  T0IF
		clrf	TMR0
		decf	cyc0,f			;    
		bsf		INTCON,T0IE		;   "TMR0"
	;  ,   
isr
		bsf		INTCON,GIE		; 
int_err							; 
		movf	05fh, w			; "STATUS"
		movwf	STATUS
		movf	05eh, w			; "W"
		retfie                  ;return from interrupt
;==========================================================================
;      RC 
			ORG   0x3FF 
           retlw   .128
;==========================================================================
; initialize eeprom locations
		
		ORG	0x2100
		DE	0x00, 0x01, 0x02, 0x03

		END                 ; directive 'end of program'
